#wav_waveform_plot
# Imports a .wav file, and plots the waveform samples
# LAST UPDATED by Andy French December 2017

#Import Python libraries
import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import tkinter as tk
from tkinter import filedialog
import wave

#

#Read sound file function. Returns left and right channel samples (16 bit integers)
#number of samples (ns), sample rate fs (Hz) and time vector (s)
def wavread( wav_filename ):

    #Open .wav file
    w = wave.open( wav_filename, 'r' )

    #Extract sample rate /Hz
    fs = w.getframerate()
    ns = w.getnframes()

    #Extract 16-bit integer numpy arrays for left and right
    #stereo channels. If original file was mono, replicate
    da = np.fromstring( w.readframes(ns), dtype=np.int16 )
    w.close()
    if w.getnchannels()==1:
        left, right = da[:], da[:]
    else:
        left, right = da[0::2], da[1::2]

    #Generate time vector
    ns = len(left)
    t = np.linspace(0, (ns-1)/fs, ns)

    #Compute duration of sound /s
    T = (ns-1)/fs

    #Return function outputs
    return left, right, ns, fs, t, T

#

#Sound export function. Adapted from: http://blog.acipo.com/wave-generation-in-python/
def wavwrite( wav_filename, ns, fs, left, right  ):

    #Open .wav file
    wf = wave.open( wav_filename,'w')

    #Set parameters of wf object
    wf.setnchannels(2) # stereo
    wf.setsampwidth(2) # assume 16-bit audio (and int16 left and right samples)
    wf.setframerate(fs)
    wf.setnframes(ns)

    #Write sample data
    ii = 0
    for i in range(0,ns):
        l = int(left[i])
        r = int(right[i])
        wf.writeframesraw( struct.pack('<hh', l, r ) )
        ii = ii + 1
        if ii > fs:
            print(" *",end="",flush=True)
            ii = 0
    wf.close()

#

#Display welcome message to console
print(" Python .wav importer & waveform plotter")
print(" By Dr Andrew French. January 2018")

#Choose .wav file using a windows dialogue box
root = tk.Tk()
root.withdraw()
wav_file = filedialog.askopenfilename(title = "Choose .wav file...",filetypes=[("WAV files","*.wav")])

#Import .wav file
print("\n Importing and plotting file: " + wav_file )
left, right, ns, fs, t, T = wavread( wav_file )

#

#Define fontsize
fsize = 14

#Create figure and axis
fig = plt.figure(figsize=(10, 8))

#Plot left channels
plt.subplot(2, 1, 1)
plt.plot( t,left,'-',label = "Left channel" , color = [1,0,0] )
plt.xlabel( "time /s", size=fsize );
plt.ylabel( "left sample amplitude", size=fsize );
plt.xticks( fontsize = fsize )
plt.yticks( fontsize = fsize )
plt.title( "Left and Right sound channels for \n" + wav_file, size=fsize*0.6 )
plt.legend( fontsize = fsize,loc = "upper right" )
plt.grid(True)
plt.xlim( 0, T)

#Plot right channel
plt.subplot(2, 1, 2)
plt.plot( t,right,'-',label = "Right channel", color = [0,0,1] )
plt.xlabel( "time /s", size=fsize );
plt.ylabel( "right sample amplitude", size=fsize );
plt.xticks( fontsize = fsize )
plt.yticks( fontsize = fsize )
plt.legend( fontsize = fsize,loc = "upper right" )
plt.grid(True)
plt.xlim( 0, T)

#Export PNG file
fig.savefig(wav_file.replace(".wav",".png"), dpi=300)

#Wait for user input to close terminal window
input("\n Press return key to continue ...")

#End of code
